@@ -69,6 +69,7 @@ a :py:class:`Model`.
 
 """
 from collections.abc import Iterable
+
 # Author: Nathan Trouvain at 01/06/2021 <nathan.trouvain@inria.fr>
 # Licence: MIT License
 # Copyright: Xavier Hinaut (2018) <xavier.hinaut@inria.fr>
@@ -78,50 +79,72 @@ from typing import Dict, List, Optional, Sequence, Tuple
 
 import numpy as np
 
-from reservoirpy.utils import progress, verbosity
-from reservoirpy.utils.validation import check_vector, is_mapping
-from ._utils import (_allocate_returned_states, _build_forward_sumodels,
-                     _dist_states_to_next_subgraph, to_ragged_seq_set, )
-from .graphflow import (DataDispatcher, find_entries_and_exits,
-                        get_offline_subgraphs, topological_sort, )
+from .graphflow import (
+    DataDispatcher,
+    find_entries_and_exits,
+    get_offline_subgraphs,
+    topological_sort,
+)
 from .types import GenericNode, MappedData
+from .utils import progress, verbosity
+from .utils.model_utils import (
+    _allocate_returned_states,
+    _build_forward_sumodels,
+    _dist_states_to_next_subgraph,
+    to_ragged_seq_set,
+)
+from .utils.validation import check_vector, is_mapping
 
 
-def _run_and_partial_fit(model, offlines, relations, x_seq, y_seq,
-                         stateful=True, reset=False, return_states=None,
-                         force_teachers=True):
+def _run_and_partial_fit(
+    model,
+    offlines,
+    relations,
+    x_seq,
+    y_seq,
+    warmup,
+    stateful=True,
+    reset=False,
+    return_states=None,
+    force_teachers=True,
+):
     """Run a submodel and call its partial fit function."""
 
     if not model.is_empty:
-        x_seq = {n: x for n, x in x_seq.items()
-                 if n in model.node_names}
+        x_seq = {n: x for n, x in x_seq.items() if n in model.node_names}
 
         if force_teachers:
-            y_seq = {n: y for n, y in y_seq.items()
-                     if n in [o.name for o in offlines]}
+            y_seq = {n: y for n, y in y_seq.items() if n in [o.name for o in offlines]}
         else:
             y_seq = None
 
         model._is_initialized = True
-        states = model.run(x_seq, y_seq,
-                           return_states=return_states,
-                           stateful=stateful,
-                           reset=reset)
+        states = model.run(
+            x_seq, y_seq, return_states=return_states, stateful=stateful, reset=reset
+        )
 
-        dist_states = _dist_states_to_next_subgraph(states,
-                                                    relations)
+        dist_states = _dist_states_to_next_subgraph(states, relations)
     else:
         dist_states = x_seq
 
     for node in offlines:
-        node.partial_fit(dist_states[node.name],
-                         y_seq[node.name])
+        node.partial_fit(dist_states[node.name], y_seq[node.name], warmup=warmup)
 
         return dist_states
 
 
-def forward(model: "Model",
-            x: MappedData) -> List[np.ndarray]:
+def _filter_teacher_nodes(Y):
+    if is_mapping(Y):
+        sequences = {n: v for n, v in Y.items() if not isinstance(v, GenericNode)}
+        teachers = {n: v for n, v in Y.items() if isinstance(v, GenericNode)}
+        return sequences, teachers
+    elif isinstance(Y, GenericNode):
+        return None, Y
+    else:
+        return Y, None
+
+
+def forward(model: "Model", x: MappedData) -> List[np.ndarray]:
     """Function applied by a :py:class:`Model` instance.
 
     This function if basically a composition of the forward functions of all
@@ -166,14 +189,12 @@ def train(model: "Model", x=None, y: MappedData = None, force_teachers=True):
 
     for node in model.nodes:
         if node.is_trained_online:
-            node.train(data[node].x, data[node].y,
-                       force_teachers=force_teachers,
-                       call=False)
+            node.train(
+                data[node].x, data[node].y, force_teachers=force_teachers, call=False
+            )
 
 
-def initializer(model: "Model",
-                x: MappedData,
-                y: Optional[MappedData] = None):
+def initializer(model: "Model", x: MappedData, y: Optional[MappedData] = None):
     """Initializes a :py:class:`Model` instance at runtime, using samples of
     data to infer all :py:class:`Node` dimensions.
 
@@ -224,9 +245,12 @@ class Model(GenericNode):
     _outputs: List[GenericNode]
     _dispatcher: "DataDispatcher"
 
-    def __init__(self, nodes: Sequence[GenericNode] = None,
-                 edges: Sequence[Tuple[GenericNode, GenericNode]] = None,
-                 name: str = None):
+    def __init__(
+        self,
+        nodes: Sequence[GenericNode] = None,
+        edges: Sequence[Tuple[GenericNode, GenericNode]] = None,
+        name: str = None,
+    ):
 
         if nodes is None:
             nodes = tuple()
@@ -270,6 +294,7 @@ class Model(GenericNode):
 
     def __iand__(self, other):
         from .ops import merge
+
         return merge(self, other, inplace=True)
 
     def _check_input(self, x):
@@ -280,7 +305,8 @@ class Model(GenericNode):
                 if input_name not in x:
                     raise NameError(
                         f"Missing input data for node '{input_name}' "
-                        f"of model {self.name}.")
+                        f"of model {self.name}."
+                    )
 
             for name, value in x.items():
                 value = check_vector(value)
@@ -292,12 +318,15 @@ class Model(GenericNode):
                             f"Impossible to call node {name} in model "
                             f"{self}: node input "
                             f"dimension is (1, {self[name].input_dim}) "
-                            f"and input dimension is {value.shape}.")
+                            f"and input dimension is {value.shape}."
+                        )
         return x
 
     def _check_targets(self, y):
-        msg = "Impossible to fit/train node {} in model, {}: node output " \
-              "dimension is (1, {}) and target dimension is {}."
+        msg = (
+            "Impossible to fit/train node {} in model, {}: node output "
+            "dimension is (1, {}) and target dimension is {}."
+        )
 
         if is_mapping(y):
             for name, value in y.items():
@@ -306,27 +335,27 @@ class Model(GenericNode):
 
                 if self._is_initialized:
                     if value.shape[1] != self[name].output_dim:
-                        raise ValueError(msg.format(name,
-                                                    self,
-                                                    self[name].output_dim,
-                                                    value.shape))
+                        raise ValueError(
+                            msg.format(name, self, self[name].output_dim, value.shape)
+                        )
         elif y is not None:
             y = check_vector(y)
             if self._is_initialized:
                 for node in self.trainable_nodes:
                     if node.is_trained_online:
                         if y.shape[1] != node.output_dim:
-                            raise ValueError(msg.format(node.name,
-                                                        self,
-                                                        node.output_dim,
-                                                        y.shape))
+                            raise ValueError(
+                                msg.format(node.name, self, node.output_dim, y.shape)
+                            )
         return y
 
     def _check_if_only_online(self):
         if any([n.is_trained_offline and not n.fitted for n in self.nodes]):
-            raise RuntimeError(f"Impossible to train model {self.name} using "
-                               f"online method: model contains untrained "
-                               f"offline nodes.")
+            raise RuntimeError(
+                f"Impossible to train model {self.name} using "
+                f"online method: model contains untrained "
+                f"offline nodes."
+            )
 
     def _load_proxys(self, keep=False):
         """Save states of all nodes into their state_proxy"""
@@ -345,16 +374,14 @@ class Model(GenericNode):
             x_init = None
             if X is not None:
                 if is_mapping(X):
-                    x_init = {name: np.atleast_2d(x[0])
-                              for name, x in X.items()}
+                    x_init = {name: np.atleast_2d(x[0]) for name, x in X.items()}
                 else:
                     x_init = np.atleast_2d(X[0])
 
             y_init = None
             if Y is not None:
                 if is_mapping(Y):
-                    y_init = {name: np.atleast_2d(y[0])
-                              for name, y in Y.items()}
+                    y_init = {name: np.atleast_2d(y[0]) for name, y in Y.items()}
                 else:
                     y_init = np.atleast_2d(Y[0])
 
@@ -382,9 +409,29 @@ class Model(GenericNode):
 
         return state
 
-    def update_graph(self, new_nodes: Sequence[GenericNode],
-                     new_edges: Sequence[
-                         Tuple[GenericNode, GenericNode]]) -> "Model":
+    def _register_teachers(self, Y=None):
+        """Register all teachers provided in all targetted nodes,
+        and remove them from Y."""
+        if Y is not None:
+            if is_mapping(Y):
+                for node, value in Y.items():
+                    if isinstance(value, GenericNode):
+                        self.get_node(node)._register_teacher(value)
+
+            elif isinstance(Y, GenericNode):
+                for node in self.trainable_nodes:
+                    if node.is_trained_online:
+                        node._register_teacher(Y)
+
+    def _unregister_teachers(self):
+        for node in self.trainable_nodes:
+            node._teacher = None
+
+    def update_graph(
+        self,
+        new_nodes: Sequence[GenericNode],
+        new_edges: Sequence[Tuple[GenericNode, GenericNode]],
+    ) -> "Model":
         """Update current Model's with new nodes and edges, inplace (a copy
         is not performed).
 
@@ -406,8 +453,7 @@ class Model(GenericNode):
         self._params = {n.name: n.params for n in self._nodes}
         self._hypers = {n.name: n.hypers for n in self._nodes}
 
-        self._inputs, self._outputs = find_entries_and_exits(self._nodes,
-                                                             self._edges)
+        self._inputs, self._outputs = find_entries_and_exits(self._nodes, self._edges)
         self._nodes = topological_sort(self._nodes, self._edges, self._inputs)
         self._node_registry = {n.name: n for n in self.nodes}
 
@@ -439,8 +485,7 @@ class Model(GenericNode):
         if self._node_registry.get(name) is not None:
             return self._node_registry[name]
         else:
-            raise KeyError(f"No node named '{name}' found in "
-                           f"model {self.name}.")
+            raise KeyError(f"No node named '{name}' found in model {self.name}.")
 
     @property
     def nodes(self) -> List[GenericNode]:
@@ -521,14 +566,16 @@ class Model(GenericNode):
     @is_trainable.setter
     def is_trainable(self, value):
         """Freeze or unfreeze trainable Nodes in the Model."""
-        trainables = [n for n in self.nodes
-                      if n.is_trained_offline or n.is_trained_online]
+        trainables = [
+            n for n in self.nodes if n.is_trained_offline or n.is_trained_online
+        ]
         for node in trainables:
             node.is_trainable = value
 
     @contextmanager
-    def with_state(self, state: Dict[str, np.ndarray] = None,
-                   stateful=False, reset=False) -> "Model":
+    def with_state(
+        self, state: Dict[str, np.ndarray] = None, stateful=False, reset=False
+    ) -> "Model":
         """Modify the state of one or several Nodes in the Model
         using a context manager.
         The modification will have effect only within the context defined,
@@ -559,9 +606,11 @@ class Model(GenericNode):
             if not stateful:
                 self.reset(to_state=current_state)
         elif isinstance(state, np.ndarray):
-            raise TypeError(f"Impossible to set state of {self.name} with "
-                            f"an array. State should be a dict mapping state "
-                            f"to a Node name within the model.")
+            raise TypeError(
+                f"Impossible to set state of {self.name} with "
+                f"an array. State should be a dict mapping state "
+                f"to a Node name within the model."
+            )
         else:
             if state is None:
                 state = {}
@@ -569,14 +618,15 @@ class Model(GenericNode):
             with ExitStack() as stack:
                 for node in self.nodes:
                     value = state.get(node.name)
-                    stack.enter_context(node.with_state(value,
-                                                        stateful=stateful,
-                                                        reset=reset))
+                    stack.enter_context(
+                        node.with_state(value, stateful=stateful, reset=reset)
+                    )
                 yield self
 
     @contextmanager
-    def with_feedback(self, feedback: Dict[str, np.ndarray] = None,
-                      stateful=False, reset=False) -> "Model":
+    def with_feedback(
+        self, feedback: Dict[str, np.ndarray] = None, stateful=False, reset=False
+    ) -> "Model":
         """Modify the feedback received or sent by Nodes in the Model using
         a context manager.
         The modification will have effect only within the context defined,
@@ -614,9 +664,9 @@ class Model(GenericNode):
             if feedback is not None:
                 for node in self.nodes:
                     value = feedback.get(node.name)
-                    stack.enter_context(node.with_feedback(value,
-                                                           stateful=stateful,
-                                                           reset=reset))
+                    stack.enter_context(
+                        node.with_feedback(value, stateful=stateful, reset=reset)
+                    )
             yield self
 
     def reset(self, to_state: Dict[str, np.ndarray] = None):
@@ -675,10 +725,15 @@ class Model(GenericNode):
             if node._buffers_initializer is not None:
                 node.initialize_buffers()
 
-    def call(self, x: MappedData, forced_feedback: MappedData = None,
-             from_state: Dict[str, np.ndarray] = None, stateful=True,
-             reset=False,
-             return_states: Sequence[str] = None) -> MappedData:
+    def call(
+        self,
+        x: MappedData,
+        forced_feedback: MappedData = None,
+        from_state: Dict[str, np.ndarray] = None,
+        stateful=True,
+        reset=False,
+        return_states: Sequence[str] = None,
+    ) -> MappedData:
         """Call the Model forward function on a single step of data.
         Model forward function is a composition of all its Nodes forward
         functions.
@@ -717,15 +772,14 @@ class Model(GenericNode):
         if not self._is_initialized:
             self.initialize(x)
 
-        # load current states in proxys interfaces accessible
-        # through feedback. These proxys are not updated during the graph call.
-        self._load_proxys()
-
         with self.with_state(from_state, stateful=stateful, reset=reset):
+            # load current states in proxys interfaces accessible
+            # through feedback. These proxys are not updated during the
+            # graph call.
+            self._load_proxys(keep=True)
+
             # maybe load forced feedbacks in proxys ?
-            with self.with_feedback(forced_feedback,
-                                    stateful=stateful,
-                                    reset=reset):
+            with self.with_feedback(forced_feedback, stateful=stateful, reset=reset):
                 state = self._call(x, return_states)
 
         # wash states proxys
@@ -733,11 +787,16 @@ class Model(GenericNode):
 
         return state
 
-    def run(self, X: MappedData = None,
-            forced_feedbacks: Dict[str, np.ndarray] = None,
-            from_state: Dict[str, np.ndarray] = None,
-            stateful=True, reset=False, shift_fb=True,
-            return_states: Sequence[str] = None) -> MappedData:
+    def run(
+        self,
+        X: MappedData = None,
+        forced_feedbacks: Dict[str, np.ndarray] = None,
+        from_state: Dict[str, np.ndarray] = None,
+        stateful=True,
+        reset=False,
+        shift_fb=True,
+        return_states: Sequence[str] = None,
+    ) -> MappedData:
         """Run the Model forward function on a sequence of data.
         Model forward function is a composition of all its Nodes forward
         functions.
@@ -782,12 +841,16 @@ class Model(GenericNode):
 
         seq = progress(
             self._dispatcher.dispatch(X, forced_feedbacks, shift_fb=shift_fb),
-            f"Running {self.name}", total=len(X))
+            f"Running {self.name}",
+            total=len(X),
+        )
 
         with self.with_state(from_state, stateful=stateful, reset=reset):
+            # load proxys after state update to make it have an
+            # impact on feedback
+            self._load_proxys(keep=True)
             for i, (x, forced_feedback, _) in enumerate(seq):
 
-                self._load_proxys()
                 with self.with_feedback(forced_feedback):
                     state = self._call(x, return_states=return_states)
 
@@ -797,6 +860,8 @@ class Model(GenericNode):
                 else:
                     states[self.output_nodes[0].name][i, :] = state
 
+                self._load_proxys()
+
         self._clean_proxys()
 
         # dicts are only relevant if model has several outputs (len > 1) or
@@ -806,9 +871,17 @@ class Model(GenericNode):
 
         return states
 
-    def train(self, X, Y=None, force_teachers=True, learn_every=1,
-              from_state=None, stateful=True,
-              reset=False, return_states=None) -> MappedData:
+    def train(
+        self,
+        X,
+        Y=None,
+        force_teachers=True,
+        learn_every=1,
+        from_state=None,
+        stateful=True,
+        reset=False,
+        return_states=None,
+    ) -> MappedData:
         """Train all online Nodes in the Model
         using their online learning rule.
 
@@ -861,17 +934,29 @@ class Model(GenericNode):
 
         self._check_if_only_online()
 
-        self._initialize_on_sequence(X, Y)
+        filt_Y, teacher_nodes = _filter_teacher_nodes(Y)
+
+        self._initialize_on_sequence(X, filt_Y)
+
+        self._register_teachers(teacher_nodes)
 
         states = _allocate_returned_states(self, X, return_states)
 
         seq_len = X.shape[0]
-        dispatch = self._dispatcher.dispatch(X, Y, return_targets=True)
-        seq = progress(dispatch, f"Training {self.name}", total=seq_len) \
-            if seq_len > 1 else dispatch
+        dispatch = self._dispatcher.dispatch(
+            X, filt_Y, return_targets=True, force_teachers=force_teachers
+        )
+
+        seq = (
+            progress(dispatch, f"Training {self.name}", total=seq_len)
+            if seq_len > 1
+            else dispatch
+        )
 
-        self._load_proxys()
         with self.with_state(from_state, stateful=stateful, reset=reset):
+            # load proxys after state update to make it have an
+            # impact on feedback
+            self._load_proxys(keep=True)
             for i, (x, forced_feedback, y) in enumerate(seq):
 
                 if not force_teachers:
@@ -881,7 +966,7 @@ class Model(GenericNode):
                     state = self._call(x, return_states=return_states)
 
                 # reload feedbacks from training. Some nodes may need
-                # feedback signals to train.
+                # updated feedback signals to train.
                 self._load_proxys()
 
                 y = self._check_targets(y)
@@ -896,6 +981,7 @@ class Model(GenericNode):
                     states[self.output_nodes[0].name][i, :] = state
 
         self._clean_proxys()
+        self._unregister_teachers()
 
         # dicts are only relevant if model has several outputs (len > 1) or
         # if we want to return states from hidden nodes
@@ -904,22 +990,32 @@ class Model(GenericNode):
 
         return states
 
-    def fit(self, X: MappedData, Y: MappedData,
-            force_teachers=True,
-            from_state=None, stateful=True, reset=False) -> "Model":
+    def fit(
+        self,
+        X: MappedData,
+        Y: MappedData,
+        warmup=0,
+        force_teachers=True,
+        from_state=None,
+        stateful=True,
+        reset=False,
+    ) -> "Model":
         """Fit all offline Nodes in the Model
         using their offline learning rule.
 
         Parameters
         ----------
-        X : numpy.ndarray or dict.
+        X : array-like or dict of array-like of shape ([series], timesteps, features)
             Input sequence of data. If dict, then pairs
             of keys and values, where keys are Model input
             nodes names and values are sequence of input data.
-        Y : numpy.ndarray or dict.
+        Y : array-like or dict of array-like of shape ([series], timesteps, features)
             Target sequence of data. If dict, then pairs
             of keys and values, where keys are Model input
             nodes names and values are sequence of target data.
+        warmup : int, default to 0
+            Number of timesteps to consider as warmup and
+            discard at the begining of each timeseries before training.
         force_teachers : bool, default to True
             If True, this Model will broadcast the available ground truth
             signal
@@ -927,7 +1023,7 @@ class Model(GenericNode):
             the real state of these nodes will be sent to the feedback
             receivers
             during training.
-        from_state : dict
+        from_state : dict, optional
             Pairs of keys and values, where keys are Model nodes names and
             value are new ndarray state vectors.
         stateful : bool, default to True
@@ -942,8 +1038,10 @@ class Model(GenericNode):
         """
 
         if not any([n for n in self.trainable_nodes if n.is_trained_offline]):
-            raise TypeError(f"Impossible to fit model {self} offline: "
-                            "no offline nodes found in model.")
+            raise TypeError(
+                f"Impossible to fit model {self} offline: "
+                "no offline nodes found in model."
+            )
 
         X, Y = to_ragged_seq_set(X), to_ragged_seq_set(Y)
         data = list(self._dispatcher.dispatch(X, Y, shift_fb=False))
@@ -953,17 +1051,14 @@ class Model(GenericNode):
         self._initialize_on_sequence(X[0], Y[0])
         self.initialize_buffers()
 
-        subgraphs = get_offline_subgraphs(self.nodes,
-                                          self.edges)
+        subgraphs = get_offline_subgraphs(self.nodes, self.edges)
 
         trained = set()
         next_X = None
 
         with self.with_state(from_state, reset=reset, stateful=stateful):
             for i, ((nodes, edges), relations) in enumerate(subgraphs):
-                submodel, offlines = _build_forward_sumodels(nodes,
-                                                             edges,
-                                                             trained)
+                submodel, offlines = _build_forward_sumodels(nodes, edges, trained)
 
                 if next_X is not None:
                     for j in range(len(X)):
@@ -977,14 +1072,17 @@ class Model(GenericNode):
                 next_X = []
                 seq = progress(X, f"Running {self.name}")
 
-                _partial_fit_fn = partial(_run_and_partial_fit,
-                                          force_teachers=force_teachers,
-                                          model=submodel,
-                                          reset=reset,
-                                          offlines=offlines,
-                                          relations=relations,
-                                          stateful=stateful,
-                                          return_states=return_states)
+                _partial_fit_fn = partial(
+                    _run_and_partial_fit,
+                    force_teachers=force_teachers,
+                    model=submodel,
+                    warmup=warmup,
+                    reset=reset,
+                    offlines=offlines,
+                    relations=relations,
+                    stateful=stateful,
+                    return_states=return_states,
+                )
 
                 # for seq/batch in dataset
                 for x_seq, y_seq in zip(seq, Y):
@@ -1009,5 +1107,7 @@ class FrozenModel(Model):
         super(FrozenModel, self).__init__(*args, **kwargs)
 
     def update_graph(self, new_nodes, new_edges):
-        raise TypeError(f"Impossible to update FrozenModel {self}: "
-                        f"model is frozen and cannot be modified.")
+        raise TypeError(
+            f"Impossible to update FrozenModel {self}: "
+            f"model is frozen and cannot be modified."
+        )
